@@ -1,21 +1,23 @@
 """The Python Anchor client."""
-from anchorpy.provider import Provider, Wallet, SendTxRequest
-from anchorpy.coder.coder import Coder, InstructionCoder, EventCoder, AccountsCoder
-from anchorpy.idl import Idl, IdlProgramAccount
-from anchorpy.workspace import create_workspace, close_workspace, WorkspaceType
-from anchorpy.program.core import Program
+from anchorpy_core.idl import Idl
+
+from anchorpy import error, utils
+from anchorpy.coder.coder import AccountsCoder, Coder, EventCoder, InstructionCoder
+from anchorpy.idl import IdlProgramAccount
 from anchorpy.program.common import (
     Event,
-    Instruction,
+    NamedInstruction,
     translate_address,
     validate_accounts,
 )
 from anchorpy.program.context import Context
-from anchorpy.program.namespace.account import AccountClient, ProgramAccount
+from anchorpy.program.core import Program
 from anchorpy.program.event import EventParser
+from anchorpy.program.namespace.account import AccountClient, ProgramAccount
 from anchorpy.program.namespace.simulate import SimulateResponse
+from anchorpy.provider import Provider, SendTxRequest, Wallet
 from anchorpy.pytest_plugin import localnet_fixture, workspace_fixture
-from anchorpy import error, utils
+from anchorpy.workspace import WorkspaceType, close_workspace, create_workspace
 
 __all__ = [
     "Program",
@@ -33,7 +35,7 @@ __all__ = [
     "InstructionCoder",
     "EventCoder",
     "AccountsCoder",
-    "Instruction",
+    "NamedInstruction",
     "IdlProgramAccount",
     "Event",
     "translate_address",
@@ -47,4 +49,4 @@ __all__ = [
 ]
 
 
-__version__ = "0.10.0"
+__version__ = "0.15.0"
