@@ -1,29 +1,30 @@
 """Extensions to the Borsh spec for Solana-specific types."""
-from typing import Any, Dict, Type, TypeVar, cast
-from keyword import kwlist
 from dataclasses import asdict
-from borsh_construct import CStruct, U8
-from solana import publickey
+from keyword import kwlist
+from typing import Any, Dict, Type, TypeVar, cast
+
+from borsh_construct import U8, CStruct
 from construct import (
-    Bytes,
     Adapter,
-    Container,
-    Padding,
+    Bytes,
     Construct,
+    Container,
     IfThenElse,
-    Switch,
+    Padding,
     Renamed,
+    Switch,
 )
+from solders import pubkey
 
 
 class BorshPubkeyAdapter(Adapter):
     def __init__(self) -> None:
         super().__init__(Bytes(32))  # type: ignore
 
-    def _decode(self, obj: bytes, context, path) -> publickey.PublicKey:
-        return publickey.PublicKey(obj)
+    def _decode(self, obj: bytes, context, path) -> pubkey.Pubkey:
+        return pubkey.Pubkey(obj)
 
-    def _encode(self, obj: publickey.PublicKey, context, path) -> bytes:
+    def _encode(self, obj: pubkey.Pubkey, context, path) -> bytes:
         return bytes(obj)
 
 
@@ -32,7 +33,7 @@ class EnumForCodegen(Adapter):
     _value_key = "value"
 
     def __init__(self, *variants: "Renamed[CStruct, CStruct]") -> None:
-        """Init enum."""  # noqa: DAR101
+        """Init enum."""
         switch_cases: dict[int, "Renamed[CStruct, CStruct]"] = {}
         variant_name_to_index: dict[str, int] = {}
         index_to_variant_name: dict[int, str] = {}
@@ -102,7 +103,7 @@ class _DataclassStruct(Adapter):
         super().__init__(cstruct)  # type: ignore
         self.datacls = datacls
 
-    def _decode(self, obj: Container, context, path) -> T:
+    def _decode(self, obj: Container, context, path) -> T:  # type: ignore
         kwargs = {}
         for key, value in obj.items():
             if key[0] != "_":
@@ -116,5 +117,5 @@ class _DataclassStruct(Adapter):
         return asdict(obj)
 
 
-BorshPubkey = BorshPubkeyAdapter()  # noqa: WPS462
-"""Adapter for (de)serializing a public key."""  # noqa: WPS322
+BorshPubkey = BorshPubkeyAdapter()
+"""Adapter for (de)serializing a public key."""
