@@ -1,47 +1,52 @@
 from pathlib import Path
-from black import format_str, FileMode
-from autoflake import fix_code
-from pyheck import snake
-from genpy import (
-    FromImport,
-    Import,
-    Assign,
-    Suite,
-    Collection,
-    ImportAs,
-    Return,
-    For,
-    If,
-    Raise,
-    Statement,
-)
-from anchorpy.coder.accounts import _account_discriminator
-from anchorpy.idl import (
+from typing import cast
+
+from anchorpy_core.idl import (
     Idl,
-    _IdlAccountDef,
+    IdlField,
+    IdlTypeDefinition,
+    IdlTypeDefinitionTyStruct,
+)
+from autoflake import fix_code
+from black import FileMode, format_str
+from genpy import (
+    Assign,
+    Collection,
+    For,
+    FromImport,
+    If,
+    Import,
+    ImportAs,
+    Raise,
+    Return,
+    Statement,
+    Suite,
+)
+from pyheck import snake
+
+from anchorpy.clientgen.common import (
+    _field_from_decoded,
+    _field_from_json,
+    _field_to_json,
+    _idl_type_to_json_type,
+    _json_interface_name,
+    _layout_for_type,
+    _py_type_from_idl,
+    _sanitize,
 )
 from anchorpy.clientgen.genpy_extension import (
+    Call,
+    ClassMethod,
+    Continue,
     Dataclass,
     Method,
-    ClassMethod,
-    TypedParam,
-    TypedDict,
+    NamedArg,
     StrDict,
     StrDictEntry,
-    NamedArg,
-    Call,
-    Continue,
-)
-from anchorpy.clientgen.common import (
-    _json_interface_name,
-    _py_type_from_idl,
-    _idl_type_to_json_type,
-    _layout_for_type,
-    _field_from_decoded,
-    _field_to_json,
-    _field_from_json,
-    _sanitize
+    TypedDict,
+    TypedParam,
 )
+from anchorpy.coder.accounts import _account_discriminator
 
 
 def gen_accounts(idl: Idl, root: Path) -> None:
@@ -87,20 +92,21 @@ def gen_accounts_code(idl: Idl, accounts_dir: Path) -> dict[Path, str]:
     return res
 
 
-def gen_account_code(acc: _IdlAccountDef, idl: Idl) -> str:
+def gen_account_code(acc: IdlTypeDefinition, idl: Idl) -> str:
     base_imports = [
         Import("typing"),
         FromImport("dataclasses", ["dataclass"]),
-        FromImport("base64", ["b64decode"]),
         FromImport("construct", ["Construct"]),
-        FromImport("solana.publickey", ["PublicKey"]),
+        FromImport("solders.pubkey", ["Pubkey"]),
         FromImport("solana.rpc.async_api", ["AsyncClient"]),
         FromImport("solana.rpc.commitment", ["Commitment"]),
         ImportAs("borsh_construct", "borsh"),
         FromImport("anchorpy.coder.accounts", ["ACCOUNT_DISCRIMINATOR_SIZE"]),
         FromImport("anchorpy.error", ["AccountInvalidDiscriminator"]),
         FromImport("anchorpy.utils.rpc", ["get_multiple_accounts"]),
-        FromImport("anchorpy.borsh_extension", ["BorshPubkey", "EnumForCodegen"]),
+        FromImport(
+            "anchorpy.borsh_extension", ["BorshPubkey", "EnumForCodegen", "COption"]
+        ),
         FromImport("..program_id", ["PROGRAM_ID"]),
     ]
     imports = (
@@ -108,7 +114,8 @@ def gen_account_code(acc: _IdlAccountDef, idl: Idl) -> str:
     )
     fields_interface_params: list[TypedParam] = []
     json_interface_params: list[TypedParam] = []
-    fields = acc.type.fields
+    ty = cast(IdlTypeDefinitionTyStruct, acc.ty)
+    fields = ty.fields
     name = _sanitize(acc.name)
     json_interface_name = _json_interface_name(name)
     layout_items: list[str] = []
@@ -117,13 +124,13 @@ def gen_account_code(acc: _IdlAccountDef, idl: Idl) -> str:
     to_json_entries: list[StrDictEntry] = []
     from_json_entries: list[NamedArg] = []
     for field in fields:
-        field_name = _sanitize(field.name)
+        field_name = _sanitize(snake(field.name))
         fields_interface_params.append(
             TypedParam(
                 field_name,
                 _py_type_from_idl(
                     idl=idl,
-                    ty=field.type,
+                    ty=field.ty,
                     types_relative_imports=False,
                     use_fields_interface_for_struct=False,
                 ),
@@ -132,12 +139,12 @@ def gen_account_code(acc: _IdlAccountDef, idl: Idl) -> str:
         json_interface_params.append(
             TypedParam(
                 field_name,
-                _idl_type_to_json_type(ty=field.type, types_relative_imports=False),
+                _idl_type_to_json_type(ty=field.ty, types_relative_imports=False),
             )
         )
         layout_items.append(
             _layout_for_type(
-                idl=idl, ty=field.type, name=field_name, types_relative_imports=False
+                idl=idl, ty=field.ty, name=field_name, types_relative_imports=False
             )
         )
         init_body_assignments.append(
@@ -147,7 +154,10 @@ def gen_account_code(acc: _IdlAccountDef, idl: Idl) -> str:
             NamedArg(
                 field_name,
                 _field_from_decoded(
-                    idl=idl, ty=field, types_relative_imports=False, val_prefix="dec."
+                    idl=idl,
+                    ty=IdlField(name=snake(field.name), docs=None, ty=field.ty),
+                    types_relative_imports=False,
+                    val_prefix="dec.",
                 ),
             )
         )
@@ -171,9 +181,9 @@ def gen_account_code(acc: _IdlAccountDef, idl: Idl) -> str:
         "fetch",
         [
             TypedParam("conn", "AsyncClient"),
-            TypedParam("address", "PublicKey"),
+            TypedParam("address", "Pubkey"),
             TypedParam("commitment", "typing.Optional[Commitment] = None"),
-            TypedParam("program_id", "PublicKey = PROGRAM_ID"),
+            TypedParam("program_id", "Pubkey = PROGRAM_ID"),
         ],
         Suite(
             [
@@ -181,13 +191,13 @@ def gen_account_code(acc: _IdlAccountDef, idl: Idl) -> str:
                     "resp",
                     "await conn.get_account_info(address, commitment=commitment)",
                 ),
-                Assign("info", 'resp["result"]["value"]'),
+                Assign("info", "resp.value"),
                 If("info is None", Return("None")),
                 If(
-                    'info["owner"] != str(program_id)',
+                    "info.owner != program_id",
                     Raise('ValueError("Account does not belong to this program")'),
                 ),
-                Assign("bytes_data", 'b64decode(info["data"][0])'),
+                Assign("bytes_data", "info.data"),
                 Return("cls.decode(bytes_data)"),
             ]
         ),
@@ -202,9 +212,9 @@ def gen_account_code(acc: _IdlAccountDef, idl: Idl) -> str:
         "fetch_multiple",
         [
             TypedParam("conn", "AsyncClient"),
-            TypedParam("addresses", "list[PublicKey]"),
+            TypedParam("addresses", "list[Pubkey]"),
             TypedParam("commitment", "typing.Optional[Commitment] = None"),
-            TypedParam("program_id", "PublicKey = PROGRAM_ID"),
+            TypedParam("program_id", "Pubkey = PROGRAM_ID"),
         ],
         Suite(
             [
