@@ -1,17 +1,19 @@
 """Common utilities."""
 from dataclasses import dataclass
-from typing import Dict, Any, Union, Tuple, NamedTuple
-from construct import Container
+from typing import Any, Dict, NamedTuple, Tuple, Union, cast
 
-from solana.publickey import PublicKey
-from anchorpy.idl import (
-    _IdlAccounts,
-    _IdlInstruction,
-    _IdlAccountItem,
+from anchorpy_core.idl import (
+    IdlAccountItem,
+    IdlAccounts,
+    IdlInstruction,
 )
+from construct import Container
+from pyheck import snake
+from solders.pubkey import Pubkey
+
 from anchorpy.program.context import Accounts
 
-AddressType = Union[PublicKey, str]
+AddressType = Union[Pubkey, str]
 
 
 class Event(NamedTuple):
@@ -22,7 +24,7 @@ class Event(NamedTuple):
 
 
 @dataclass
-class Instruction:
+class NamedInstruction:
     """Container for a named instruction.
 
     Attributes:
@@ -34,7 +36,7 @@ class Instruction:
     name: str
 
 
-def _to_instruction(idl_ix: _IdlInstruction, args: Tuple) -> Instruction:
+def _to_instruction(idl_ix: IdlInstruction, args: Tuple) -> NamedInstruction:
     """Convert an IDL instruction and arguments to an Instruction object.
 
     Args:
@@ -51,11 +53,11 @@ def _to_instruction(idl_ix: _IdlInstruction, args: Tuple) -> Instruction:
         raise ValueError("Invalid argument length")
     ix: Dict[str, Any] = {}
     for idx, ix_arg in enumerate(idl_ix.args):
-        ix[ix_arg.name] = args[idx]
-    return Instruction(data=ix, name=idl_ix.name)
+        ix[snake(ix_arg.name)] = args[idx]
+    return NamedInstruction(data=ix, name=snake(idl_ix.name))
 
 
-def validate_accounts(ix_accounts: list[_IdlAccountItem], accounts: Accounts):
+def validate_accounts(ix_accounts: list[IdlAccountItem], accounts: Accounts):
     """Check that accounts passed in `ctx` match the IDL.
 
     Args:
@@ -66,21 +68,23 @@ def validate_accounts(ix_accounts: list[_IdlAccountItem], accounts: Accounts):
         ValueError: If `ctx` accounts don't match the IDL.
     """
     for acc in ix_accounts:
-        if isinstance(acc, _IdlAccounts):
-            validate_accounts(acc.accounts, accounts[acc.name])
-        elif acc.name not in accounts:
-            raise ValueError(f"Invalid arguments: {acc.name} not provided")
+        acc_name = snake(acc.name)
+        if isinstance(acc, IdlAccounts):
+            nested = cast(Accounts, accounts[acc_name])
+            validate_accounts(acc.accounts, nested)
+        elif acc_name not in accounts:
+            raise ValueError(f"Invalid arguments: {acc_name} not provided")
 
 
-def translate_address(address: AddressType) -> PublicKey:
-    """Convert `str | PublicKey` into `PublicKey`.
+def translate_address(address: AddressType) -> Pubkey:
+    """Convert `str | Pubkey` into `Pubkey`.
 
     Args:
-        address: Public key as string or `PublicKey`.
+        address: Public key as string or `Pubkey`.
 
     Returns:
-        Public key as `PublicKey`.
+        Public key as `Pubkey`.
     """
     if isinstance(address, str):
-        return PublicKey(address)
+        return Pubkey.from_string(address)
     return address
