@@ -1,25 +1,24 @@
 """Provides the `AccountClient` class."""
-import base64
 from dataclasses import dataclass
-from base58 import b58encode
-from typing import Any, Optional, Dict, List
+from typing import Any, Dict, List, Optional, Sequence, Union
 
+from anchorpy_core.idl import Idl, IdlTypeDefinition
+from based58 import b58encode
 from construct import Container
-from solana.keypair import Keypair
-from solana.system_program import create_account, CreateAccountParams
-from solana.transaction import TransactionInstruction
-from solana.publickey import PublicKey
-from solana.rpc.types import MemcmpOpts
 from solana.rpc.commitment import Commitment
+from solana.rpc.types import MemcmpOpts
+from solana.transaction import Instruction
+from solders.keypair import Keypair
+from solders.pubkey import Pubkey
+from solders.system_program import CreateAccountParams, create_account
 
-from anchorpy.coder.common import _account_size
 from anchorpy.coder.accounts import (
     ACCOUNT_DISCRIMINATOR_SIZE,
     _account_discriminator,
 )
 from anchorpy.coder.coder import Coder
+from anchorpy.coder.common import _account_size
 from anchorpy.error import AccountDoesNotExistError, AccountInvalidDiscriminator
-from anchorpy.idl import Idl, _IdlAccountDef
 from anchorpy.provider import Provider
 from anchorpy.utils.rpc import get_multiple_accounts
 
@@ -27,7 +26,7 @@ from anchorpy.utils.rpc import get_multiple_accounts
 def _build_account(
     idl: Idl,
     coder: Coder,
-    program_id: PublicKey,
+    program_id: Pubkey,
     provider: Provider,
 ) -> Dict[str, "AccountClient"]:
     """Generate the `.account` namespace.
@@ -52,7 +51,7 @@ def _build_account(
 class ProgramAccount:
     """Deserialized account owned by a program."""
 
-    public_key: PublicKey
+    public_key: Pubkey
     account: Container
 
 
@@ -62,9 +61,9 @@ class AccountClient(object):
     def __init__(
         self,
         idl: Idl,
-        idl_account: _IdlAccountDef,
+        idl_account: IdlTypeDefinition,
         coder: Coder,
-        program_id: PublicKey,
+        program_id: Pubkey,
         provider: Provider,
     ):
         """Init.
@@ -83,7 +82,7 @@ class AccountClient(object):
         self._size = ACCOUNT_DISCRIMINATOR_SIZE + _account_size(idl, idl_account)
 
     async def fetch(
-        self, address: PublicKey, commitment: Optional[Commitment] = None
+        self, address: Pubkey, commitment: Optional[Commitment] = None
     ) -> Container[Any]:
         """Return a deserialized account.
 
@@ -101,9 +100,9 @@ class AccountClient(object):
             encoding="base64",
             commitment=commitment,
         )
-        if not account_info["result"]["value"]:
+        if not account_info.value:
             raise AccountDoesNotExistError(f"Account {address} does not exist")
-        data = base64.b64decode(account_info["result"]["value"]["data"][0])
+        data = account_info.value.data
         discriminator = _account_discriminator(self._idl_account.name)
         if discriminator != data[:ACCOUNT_DISCRIMINATOR_SIZE]:
             msg = f"Account {address} has an invalid discriminator"
@@ -112,7 +111,7 @@ class AccountClient(object):
 
     async def fetch_multiple(
         self,
-        addresses: List[PublicKey],
+        addresses: List[Pubkey],
         batch_size: int = 300,
         commitment: Optional[Commitment] = None,
     ) -> list[Optional[Container[Any]]]:
@@ -147,7 +146,7 @@ class AccountClient(object):
         self,
         signer: Keypair,
         size_override: int = 0,
-    ) -> TransactionInstruction:
+    ) -> Instruction:
         """Return an instruction for creating this account.
 
         Args:
@@ -166,27 +165,25 @@ class AccountClient(object):
         return create_account(
             CreateAccountParams(
                 from_pubkey=self._provider.wallet.public_key,
-                new_account_pubkey=signer.public_key,
+                to_pubkey=signer.pubkey(),
                 space=space,
-                lamports=mbre_resp["result"],
-                program_id=self._program_id,
+                lamports=mbre_resp.value,
+                owner=self._program_id,
             )
         )
 
-    async def all(
+    async def all(  # noqa: A003
         self,
         buffer: Optional[bytes] = None,
-        memcmp_opts: Optional[list[MemcmpOpts]] = None,
-        data_size: Optional[int] = None,
+        filters: Optional[Sequence[Union[int, MemcmpOpts]]] = None,
     ) -> list[ProgramAccount]:
         """Return all instances of this account type for the program.
 
         Args:
             buffer: bytes filter to append to the discriminator.
-            memcmp_opts: Options to compare a provided series of bytes with program
-                account data at a particular offset.
-            data_size: Option to compare the program account data length with the
-                provided data size.
+            filters: (optional) Options to compare a provided series of bytes with
+                program account data at a particular offset.
+                Note: an int entry is converted to a `dataSize` filter.
         """
         all_accounts = []
         discriminator = _account_discriminator(self._idl_account.name)
@@ -196,21 +193,18 @@ class AccountClient(object):
             offset=0,
             bytes=bytes_arg,
         )
-        extra_memcmpm_opts = [] if memcmp_opts is None else memcmp_opts
-        full_memcmp_opts = [base_memcmp_opt] + extra_memcmpm_opts
+        filters_to_use = [base_memcmp_opt] + [] if filters is None else filters
         resp = await self._provider.connection.get_program_accounts(
             self._program_id,
             encoding="base64",
-            commitment=self.provider.connection._commitment,  # noqa: WPS437
-            data_size=data_size,
-            memcmp_opts=full_memcmp_opts,
+            commitment=self.provider.connection._commitment,
+            filters=filters_to_use,
         )
-        for r in resp["result"]:
-            account_data = r["account"]["data"][0]
-            account_data = base64.b64decode(account_data)
+        for r in resp.value:
+            account_data = r.account.data
             all_accounts.append(
                 ProgramAccount(
-                    public_key=PublicKey(r["pubkey"]),
+                    public_key=r.pubkey,
                     account=self._coder.accounts.decode(account_data),
                 ),
             )
@@ -222,7 +216,7 @@ class AccountClient(object):
         return self._size
 
     @property
-    def program_id(self) -> PublicKey:
+    def program_id(self) -> Pubkey:
         """Return the program ID owning all accounts."""
         return self._program_id
 
