@@ -1,7 +1,9 @@
 """This module contains code for handling user-defined types."""
-from typing import Type, Any
+from typing import Any, Type
+
+from anchorpy_core.idl import Idl
+
 from anchorpy.coder.idl import _idl_typedef_to_python_type
-from anchorpy.idl import Idl
 
 
 def _build_types(
