@@ -1,19 +1,21 @@
 """This module contains the Provider class and associated utilities."""
 from __future__ import annotations
 
-from pathlib import Path
-from os import getenv, environ
 import json
+from os import environ, getenv
+from pathlib import Path
 from types import MappingProxyType
-from typing import List, Optional, Union, NamedTuple
+from typing import List, NamedTuple, Optional, Union
 
 from more_itertools import unique_everseen
-from solana.keypair import Keypair
 from solana.rpc import types
 from solana.rpc.async_api import AsyncClient
-from solana.rpc.commitment import Finalized, Processed, Confirmed
-from solana.transaction import Transaction, TransactionSignature
-from solana.publickey import PublicKey
+from solana.rpc.commitment import Confirmed, Finalized, Processed
+from solana.transaction import Transaction
+from solders.keypair import Keypair
+from solders.pubkey import Pubkey
+from solders.rpc.responses import SimulateTransactionResp
+from solders.signature import Signature
 
 
 class SendTxRequest(NamedTuple):
@@ -93,8 +95,8 @@ class Provider:
         self,
         tx: Transaction,
         signers: Optional[list[Keypair]] = None,
-        opts: types.TxOpts = None,
-    ) -> types.RPCResponse:
+        opts: Optional[types.TxOpts] = None,
+    ) -> SimulateTransactionResp:
         """Simulate the given transaction, returning emitted logs from execution.
 
         Args:
@@ -110,10 +112,10 @@ class Provider:
             signers = []
         if opts is None:
             opts = self.opts
-        recent_blockhash_resp = await self.connection.get_recent_blockhash(
+        recent_blockhash_resp = await self.connection.get_latest_blockhash(
             Finalized,
         )
-        tx.recent_blockhash = recent_blockhash_resp["result"]["value"]["blockhash"]
+        tx.recent_blockhash = recent_blockhash_resp.value.blockhash
         tx.fee_payer = self.wallet.public_key
         all_signers = list(unique_everseen([self.wallet.payer, *signers]))
         tx.sign(*all_signers)
@@ -125,8 +127,8 @@ class Provider:
         self,
         tx: Transaction,
         signers: Optional[list[Keypair]] = None,
-        opts: types.TxOpts = None,
-    ) -> TransactionSignature:
+        opts: Optional[types.TxOpts] = None,
+    ) -> Signature:
         """Send the given transaction, paid for and signed by the provider's wallet.
 
         Args:
@@ -145,13 +147,13 @@ class Provider:
         tx.fee_payer = self.wallet.public_key
         all_signers = list(unique_everseen([self.wallet.payer, *signers]))
         resp = await self.connection.send_transaction(tx, *all_signers, opts=opts)
-        return resp["result"]
+        return resp.value
 
     async def send_all(
         self,
         reqs: list[Union[Transaction, SendTxRequest]],
         opts: Optional[types.TxOpts] = None,
-    ) -> list[TransactionSignature]:
+    ) -> list[Signature]:
         """Similar to `send`, but for an array of transactions and signers.
 
         Args:
@@ -178,12 +180,12 @@ class Provider:
             resp = await self.connection.send_raw_transaction(
                 signed.serialize(), opts=opts
             )
-            sigs.append(resp["result"])
+            sigs.append(resp.value)
         return sigs
 
     async def __aenter__(self) -> Provider:
         """Use as a context manager."""
-        await self.connection.__aenter__()  # noqa: WPS609
+        await self.connection.__aenter__()
         return self
 
     async def __aexit__(self, _exc_type, _exc, _tb):
@@ -207,9 +209,9 @@ class Wallet:
         self.payer = payer
 
     @property
-    def public_key(self) -> PublicKey:
+    def public_key(self) -> Pubkey:
         """Get the public key of the wallet."""
-        return self.payer.public_key
+        return self.payer.pubkey()
 
     def sign_transaction(self, tx: Transaction) -> Transaction:
         """Sign a transaction using the wallet's keypair.
@@ -245,11 +247,11 @@ class Wallet:
         """
         path = Path(getenv("ANCHOR_WALLET", Path.home() / ".config/solana/id.json"))
         with path.open() as f:
-            keypair = json.load(f)
-        return cls(Keypair.from_secret_key(bytes(keypair)))
+            keypair: List[int] = json.load(f)
+        return cls(Keypair.from_bytes(keypair))
 
     @classmethod
     def dummy(cls) -> Wallet:
         """Create a dummy wallet instance that won't be used to sign transactions."""
-        keypair = Keypair.from_secret_key(bytes([0] * 64))  # noqa: WPS435
+        keypair = Keypair.from_bytes([0] * 64)
         return cls(keypair)
